##############################################################################
#
#    Xbus Common, for Odoo
#    Copyright © 2023 XCG Consulting <https://xcg-consulting.fr>
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################
import base64
import uuid

from odoo import Command, exceptions

from .modelcase import ModelCase


class TestCommon(ModelCase):
    """Test the Xbus common models."""

    def test_creates(self):
        """Test adding envelope and messages from Odoo"""
        user_xbus = self.env.ref("xbus_common.user_xbus_demo")
        envelope = (
            self.env["xbus.envelope"].with_user(user_xbus).create([{"comment": ""}])
        )
        self.assertNotEqual(envelope.uid, "")
        self.assertNotEqual(envelope.create_date, "")
        messages = (
            self.env["xbus.message"]
            .with_user(user_xbus)
            .create(
                [
                    {
                        "envelope_id": envelope.id,
                        "type": "test_1",
                        "body": base64.encodebytes(b"Items"),
                        "link_ids": [
                            Command.create(
                                {"res_model": "res.users", "res_id": self.env.user.id}
                            ),
                            Command.create(
                                {"res_model": "non_existing_model", "res_id": 1}
                            ),
                            Command.create(
                                {
                                    "res_model": "res.company",
                                    "res_id": 0,
                                }
                            ),
                        ],
                    },
                    {
                        "envelope_id": envelope.id,
                        "type": "test_2",
                        "header": base64.encodebytes(b"Header"),
                        "body": base64.encodebytes(b"Body"),
                    },
                ]
            )
        )
        self.assertNotEqual(messages[0].uid, "")
        self.assertNotEqual(messages[1].uid, "")
        self.assertEqual(
            messages[0].link_ids[0].action_open_document()["res_id"], self.env.user.id
        )
        with self.assertRaises(ValueError):
            messages[0].link_ids.action_open_document()
        with self.assertRaises(exceptions.MissingError):
            messages[0].link_ids[1].action_open_document()
        with self.assertRaises(exceptions.MissingError):
            messages[0].link_ids[2].action_open_document()

    def test_external_add(self):
        """Test as if another tool wrote to the common xbus tables."""
        # Simulate someone else writing to the table, so use SQL
        # Convert uuid to string to please psycopg2
        uid = str(uuid.uuid4())
        self.cr.execute(
            "INSERT INTO xbus_envelope (comment, uid) VALUES ('test cÔmment', %s) "
            "RETURNING id",
            (uid,),
        )
        envelope_id = self.cr.fetchall()[0][0]
        envelope = self.env["xbus.envelope"].browse([envelope_id])
        self.assertEqual(envelope.uid, uid)
        self.assertNotEqual(envelope.create_date, "")
        self.assertEqual(envelope.comment, "test cÔmment")
        # Add a message with header and body
        uid = str(uuid.uuid4())
        self.cr.execute(
            "INSERT INTO xbus_message (envelope_id, uid, type, header, body) VALUES "
            "(%s, %s, %s, %s, %s) RETURNING id",
            (envelope_id, uid, "test_1", b"header", b"body"),
        )
        message1_id = self.cr.fetchall()[0][0]
        # Add a message with body only
        uid = str(uuid.uuid4())
        self.cr.execute(
            "INSERT INTO xbus_message (envelope_id, uid, type, body) VALUES "
            "(%s, %s, %s, %s) RETURNING id",
            (envelope_id, uid, "test_2", b"body"),
        )
        message2_id = self.cr.fetchall()[0][0]
        self.assertNotEqual(message1_id, message2_id)
        messages = (
            self.env["xbus.message"]
            .with_user(self.env.ref("base.user_admin"))
            .search([("envelope_id", "=", envelope_id)])
        )
        self.assertEqual(len(messages), 2)
